-- keep this file horizontally line-aligned with w3i-write|read

require("wc3-packhelper-library@4355d8d/wc3-read")
require("w3u-common")

-- takes file handle to .w3u at offset 0
-- reads and parses values, inserting into a table
-- (file handle must be closed by caller)
-- returns table
function parseW3u(file, fileExtension)
	
	assert(file)
	assert(isValidFormat(fileExtension),
		"File is of unknown format: '".. tostring(fileExtension) .."'. "..
		"It must be one of: '".. table.concat(W3U_FORMAT, ",") .."'"
	)
	local w3u = {}
	w3u.w3u_converter_file_extension = fileExtension
	w3u.version = readIntU(file)
	
	
	
	
	
	
	
	w3u.default_object_modified_count = readIntU(file)
	w3u.default_object_modified = 
		parseW3uObjectTable(file, fileExtension,
			w3u.default_object_modified_count)
	
	
	
	
	
	
	
	
	w3u.custom_object_modified_count = readIntU(file)
	w3u.custom_object_modified = 
		parseW3uObjectTable(file, fileExtension, w3u.custom_object_modified_count)



	-- sanity check if EOF reached
	local fileCurPos = file:seek("cur")
	local fileEndPos = file:seek("end")
	if fileCurPos ~= fileEndPos then
		error("Finished parsing, but there's more data in the file! " ..
			"Expected EOF at ".. fileCurPos ..
			"Actual EOF at ".. fileEndPos)
	end

    return w3u
end

-- returns object table
function parseW3uObjectTable(file, extension, objCount)
	local tbl = {}
	
	for i = 0, objCount-1 do
		local obj = {}
		tbl[i] = obj
		
		obj.rawcode_default = readRawcode(file)
		obj.rawcode_custom = readRawcode(file)
		
		
		
		
		
		
		
		obj.fields_modified_count = readIntU(file)
		obj.fields_modified = {}
		
		for f = 0, obj.fields_modified_count-1 do
		
			local fieldName = readRawcode(file)
			local fieldData = {}
			obj.fields_modified[fieldName] = fieldData
			
			fieldData.type = readIntU(file)
			
			-- only for usability, custom
			if fieldData.type == 0 then
				fieldData.type_explanation = "integer"
			elseif fieldData.type == 1 then
				fieldData.type_explanation = "float"
			elseif fieldData.type == 2 then
				fieldData.type_explanation = "float"
			elseif fieldData.type == 3 then
				fieldData.type_explanation = "string"
			else
				fieldData.type_explanation = "unknown! please report"
			end
			--
			
			if isExtensionW3adq(extension) then
				if extension == "w3a" or extension == "w3q" then
					fieldData.level = readIntU(file)
				elseif extension == "w3d" then
					fieldData.variant = readIntU(file)
				else
					-- nothing
				end
				
				fieldData.letter_code = readIntU(file)
				
				if fieldData.letter_code == 0 then
					fieldData.letter_code_explanation = "normal"
				elseif fieldData.letter_code > 0 and fieldData.letter_code <= 26 then
					-- e.g. "DataA"
					fieldData.letter_code_explanation = "Data" .. string.char(64 + fieldData.letter_code)
				else
					fieldData.letter_code_explanation = "unknown! please report"
				end
			end
			
			if fieldData.type == 0 then
				fieldData.value = readIntU(file)
			elseif fieldData.type == 1 then
				fieldData.value = readFloat(file)
			elseif fieldData.type == 2 then
				fieldData.value = readFloat(file)
			elseif fieldData.type == 3 then
				fieldData.value = readString(file)
			else
				error("Unknown field data type: ".. tostring(fieldData.type))
			end
			
			fieldData.rawcode_unused = readRawcode(file)
		end
	end
	
	return tbl
end
