#!/usr/bin/env lua

require("w3u-read")
require("w3u-write")

local args = args or {...}
function main(args)
	assert(#args == 1 or #args == 2, "You must specify one or two arguments. Received arguments: ".. #args)
	
	local inputMode;
	local outputMode;
	local fileName;
	
	if args[1] == "--from-json" then
		inputMode = "json"
		outputMode = "war3"
		fileName = args[2]
		
	elseif args[1] == "--from-lua" then
		inputMode = "lua"
		outputMode = "war3"
		fileName = args[2]
		
	elseif args[1] == "--to-json" then
		inputMode = "war3"
		outputMode = "json"
		fileName = args[2]
		
	elseif args[1] == "--to-lua" then
		inputMode = "war3"
		outputMode = "lua"
		fileName = args[2]
		
	elseif args[1] == "--help" or args[1] == "-h" then
		io.stderr:write([[
This program converts Warcraft's .w3u, .w3t, .w3b, .w3d, .w3a, .w3h, .w3q files to JSON/Lua for editing and back.
The output is printed to STDOUT and must be redirected to be saved to file.

Options:
--from-json <json data file> - converts to a war3 file
--from-lua <lua data file> - converts to a war3 file

--to-json <war3 data file> - converts to a JSON data file
--to-lua <war3 data file> - converts to a Lua data file

Usage example:
lua w3u-run.lua --to-lua war3map.w3u > unit-data.lua

lua w3u-run.lua --from-lua unit-data.lua > war3map-modified.w3u
]])
		
	else
		error("Unknown argument: '".. tostring(args[1]) .."'. Use --help to see help")
	end
	io.stderr:write("Input mode = ".. tostring(inputMode) .."\n"
		.."Output mode = ".. tostring(outputMode) .."\n"
		.."File name = ".. tostring(fileName) .."\n"
	)
	assert(fileName, "You must specify a file as second argument!")
	
	local w3uData;
	local luaData, jsonData;
	if inputMode == "war3" then
		local w3uFile = assert(io.open(fileName, "rb"))
		local ext = fileName:match("%.(w3%w)$")
		w3uData = parseW3u(w3uFile, ext)
		w3uFile:close()
		
		io.stderr:write("w3u returned: ".. tostring(w3uData), "\n")
	
	elseif inputMode == "json" then
		local JSON = require"lib/JSON"
		
		local jsonFile = assert(io.open(fileName, "rb"))
		-- the issue with JSON: numeric keys will be converted to string keys
		-- if a table contains both number+string keys. Let the parser deal with it.
		jsonData = JSON:decode(jsonFile:read("*a"))
		jsonFile:close()
	elseif inputMode == "lua" then
		luaData = dofile(fileName)
	end
	
	if outputMode == "json" then
		local JSON = require"lib/JSON"
		local jsonText = JSON:encode_pretty(w3uData)
		io.stdout:write(jsonText)
		
	elseif outputMode == "lua" then
		local serpent = require"lib/serpent@0-30/src/serpent"
		local luaText = serpent.block(w3uData)
		io.stdout:write("return\n")
		io.stdout:write(luaText)
	
	elseif outputMode == "war3" then
		-- that's fine as long as it doesn't do seek
		writeW3u(io.stdout, jsonData or luaData)
	end
end
main(args)