-- keep this file horizontally line-aligned with w3i-write|read

require("wc3-packhelper-library@4355d8d/wc3-write")
require("w3u-common")

-- takes file handle to .w3u at offset 0
-- reads and parses values, inserting into a table
-- (file handle must be closed by caller)
-- returns table
function writeW3u(file, w3u)
	local fileExtension = w3u.w3u_converter_file_extension
	assert(file)
	assert(isValidFormat(fileExtension),
		"File is of unknown format: '".. tostring(fileExtension) .."'. "..
		"It must be one of: '".. table.concat(W3U_FORMAT, ",") .."'"
	)
	
	
	writeIntU(file, w3u.version)
	
	-- w3u.default_object_modified_count
	local count = 0
	for k, v in pairs(w3u.default_object_modified) do
		--io.stderr:write(type(k),"->",type(v),"::",tostring(k),"=",tostring(v),"\n")
		count = count + 1
	end
	w3u.default_object_modified_count = count
	writeIntU(file, w3u.default_object_modified_count) 
	writeW3uObjectTable(file, fileExtension,
		w3u.default_object_modified, w3u.default_object_modified_count)
	
	
	-- w3u.custom_object_modified_count
	local count = 0
	for k, v in pairs(w3u.custom_object_modified) do
		--io.stderr:write(type(k),"->",type(v),"::",tostring(k),"=",tostring(v),"\n")
		count = count + 1
	end
	w3u.custom_object_modified_count = count
	writeIntU(file, w3u.custom_object_modified_count) 
	writeW3uObjectTable(file, fileExtension,
		w3u.custom_object_modified, w3u.custom_object_modified_count)
	

	
	
	
	
	
	
	
	

    return w3u
end

-- returns object table
function writeW3uObjectTable(file, extension, objArray, objCount)
	
	
	for i = 0, objCount-1 do
		--io.stderr:write("Reading object, i = ".. i, "\n")
		local obj = objArray[i] or objArray[tostring(i)] -- lua / json key

		
		writeRawcode(file, obj.rawcode_default)
		writeRawcode(file, obj.rawcode_custom)
		
		local counted = 0
		for _, __ in pairs(obj.fields_modified) do
			counted = counted + 1
		end
		obj.fields_modified_count = counted
		
		writeIntU(file, obj.fields_modified_count)
		
		
		for fieldName, fieldData in pairs(obj.fields_modified) do
		
			writeRawcode(file, fieldName)
			
			
			
			writeIntU(file, fieldData.type)
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			if isExtensionW3adq(extension) then
				if extension == "w3a" or extension == "w3q" then
					writeIntU(file, fieldData.level)
				elseif extension == "w3d" then
					writeIntU(file, fieldData.variant)
				else
					-- nothing
				end
				
				writeIntU(file, fieldData.letter_code)
				
				
				
				
				
				
				
				
				
			end
			
			if fieldData.type == 0 then
				writeIntU(file, fieldData.value)
			elseif fieldData.type == 1 then
				writeFloat(file, fieldData.value)
			elseif fieldData.type == 2 then
				writeFloat(file, fieldData.value)
			elseif fieldData.type == 3 then
				writeString(file, fieldData.value)
			else
				error("Unknown field data type: ".. tostring(fieldData.type))
			end
			
			writeRawcode(file, fieldData.rawcode_unused)
		end
	end
	
	
end
